@@ -59,6 +59,16 @@ type ContentType = S.ByteString
 -- construct from a URL, and then use the records below to make modifications.
 -- This approach allows http-conduit to add configuration options without
 -- breaking backwards compatibility.
+--
+-- For example, to construct a POST request, you could do something like:
+--
+-- > initReq <- parseUrl "http://www.example.com/path"
+-- > let req = initReq
+-- >             { method = "POST"
+-- >             }
+--
+-- For more information, please see
+-- <http://www.yesodweb.com/book/settings-types>.
 data Request m = Request
     { method :: W.Method
     -- ^ HTTP request method, eg GET, POST.
@@ -116,6 +126,7 @@ data Proxy = Proxy
 
 encodeUrlCharPI :: Char -> String
 encodeUrlCharPI '/' = "/"
+encodeUrlCharPI '%' = "%"
 encodeUrlCharPI c = encodeUrlChar c
 
 encodeUrlChar :: Char -> String
@@ -214,8 +225,11 @@ parseUrl2 full sec s = do
 data HttpException = StatusCodeException W.Status W.ResponseHeaders
                    | InvalidUrlException String String
                    | TooManyRedirects
+                   | UnparseableRedirect
+                   | TooManyRetries
                    | HttpParserException String
                    | HandshakeFailed
+                   | OverlongHeaders
     deriving (Show, Typeable)
 instance Exception HttpException
 
@@ -273,7 +287,7 @@ needsGunzip req hs' =
      && decompress req (fromMaybe "" $ lookup "content-type" hs')
 
 requestBuilder
-    :: C.Resource m
+    :: Monad m
     => Request m
     -> C.Source m Builder
 requestBuilder req =
@@ -311,11 +325,6 @@ requestBuilder req =
     builder =
             fromByteString (method req)
             <> fromByteString " "
-            <> (case proxy req of
-                    Just{} ->
-                        fromByteString (if secure req then "https://" else "http://")
-                        <> fromByteString hh
-                    Nothing -> mempty)
             <> (case S8.uncons $ path req of
                     Just ('/', _) -> fromByteString $ path req
                     _ -> fromByteString "/" <> fromByteString (path req))
