@@ -12,17 +12,23 @@ module Network.HTTP.Conduit.Response
 import Control.Arrow (first)
 import Data.Typeable (Typeable)
 import Data.Monoid (mempty)
+import Control.Monad (liftM)
+
+import Control.Exception (throwIO)
+import Control.Monad.IO.Class (liftIO)
 
 import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as L
 
 import qualified Data.CaseInsensitive as CI
 
-import Control.Monad.Trans.Resource (ResourceT, ResourceIO)
+import Control.Monad.Trans.Resource (MonadResource)
+import Control.Monad.Trans.Class (lift)
 import qualified Data.Conduit as C
 import qualified Data.Conduit.Zlib as CZ
 import qualified Data.Conduit.Binary as CB
 import qualified Data.Conduit.List as CL
+import qualified Data.Conduit.Internal
 
 import qualified Network.HTTP.Types as W
 
@@ -32,9 +38,12 @@ import Network.HTTP.Conduit.Util
 import Network.HTTP.Conduit.Parser
 import Network.HTTP.Conduit.Chunk
 
+import Data.Void (absurd)
+
 -- | A simple representation of the HTTP response created by 'lbsConsumer'.
 data Response body = Response
-    { statusCode :: W.Status
+    { responseStatus :: W.Status
+    , responseVersion :: W.HttpVersion
     , responseHeaders :: W.ResponseHeaders
     , responseBody :: body
     }
@@ -42,7 +51,7 @@ data Response body = Response
 
 -- | Since 1.1.2.
 instance Functor Response where
-    fmap f (Response status headers body) = Response status headers (f body)
+    fmap f (Response status v headers body) = Response status v headers (f body)
 
 -- | If a request is a redirection (status code 3xx) this function will create
 -- a new request from the old request, the server headers returned with the
@@ -94,9 +103,9 @@ getRedirectedRequest req hs code
 
 -- | Convert a 'Response' that has a 'C.Source' body to one with a lazy
 -- 'L.ByteString' body.
-lbsResponse :: C.Resource m
-            => ResourceT m (Response (C.Source m S8.ByteString))
-            -> ResourceT m (Response L.ByteString)
+lbsResponse :: Monad m
+            => m (Response (C.Source m S8.ByteString))
+            -> m (Response L.ByteString)
 lbsResponse mres = do
     res <- mres
     bss <- responseBody res C.$$ CL.consume
@@ -104,58 +113,56 @@ lbsResponse mres = do
         { responseBody = L.fromChunks bss
         }
 
-getResponse :: ResourceIO m
+checkHeaderLength :: MonadResource m
+                  => Int
+                  -> C.Sink S8.ByteString m a
+                  -> C.Sink S8.ByteString m a
+checkHeaderLength len C.NeedInput{}
+    | len <= 0 =
+        let x = liftIO $ throwIO OverlongHeaders
+         in C.PipeM x (lift x)
+checkHeaderLength len (C.NeedInput pushI closeI) = C.NeedInput
+    (\bs -> checkHeaderLength
+        (len - S8.length bs)
+        (pushI bs)) closeI
+checkHeaderLength len (C.PipeM msink close) = C.PipeM (liftM (checkHeaderLength len) msink) close
+checkHeaderLength _ s@C.Done{} = s
+checkHeaderLength _ (C.HaveOutput _ _ o) = absurd o
+
+getResponse :: MonadResource m
             => ConnRelease m
             -> Request m
-            -> C.BufferedSource m S8.ByteString
-            -> ResourceT m (Response (C.Source m S8.ByteString))
-getResponse connRelease req@(Request {..}) bsrc = do
-    ((_, sc, sm), hs) <- bsrc C.$$ sinkHeaders
+            -> C.Source m S8.ByteString
+            -> m (Response (C.Source m S8.ByteString))
+getResponse connRelease req@(Request {..}) src1 = do
+    (src2, ((vbs, sc, sm), hs)) <- src1 C.$$+ checkHeaderLength 4096 sinkHeaders
+    let version = if vbs == "1.1" then W.http11 else W.http10
     let s = W.Status sc sm
     let hs' = map (first CI.mk) hs
     let mcl = lookup "content-length" hs' >>= readDec . S8.unpack
 
     -- should we put this connection back into the connection manager?
     let toPut = Just "close" /= lookup "connection" hs'
-    let cleanup = connRelease $ if toPut then Reuse else DontReuse
+    let cleanup bodyConsumed = connRelease $ if toPut && bodyConsumed then Reuse else DontReuse
 
     -- RFC 2616 section 4.4_1 defines responses that must not include a body
     body <-
         if hasNoBody method sc || mcl == Just 0
             then do
-                cleanup
+                cleanup True
                 return mempty
             else do
-                let bsrc' =
+                let src3 =
                         if ("transfer-encoding", "chunked") `elem` hs'
-                            then bsrc C.$= chunkedConduit rawBody
+                            then src2 C.$= chunkedConduit rawBody
                             else
                                 case mcl of
-                                    Just len -> bsrc C.$= CB.isolate len
-                                    Nothing  -> C.unbufferSource bsrc
-                let bsrc'' =
+                                    Just len -> src2 C.$= CB.isolate len
+                                    Nothing  -> src2
+                let src4 =
                         if needsGunzip req hs'
-                            then bsrc' C.$= CZ.ungzip
-                            else bsrc'
-                return $ addCleanup cleanup bsrc''
+                            then src3 C.$= CZ.ungzip
+                            else src3
+                return $ Data.Conduit.Internal.addCleanup cleanup src4
 
-    return $ Response s hs' body
-
--- | Add some cleanup code to the given 'C.Source'. General purpose
--- function, could be included in conduit itself.
-addCleanup :: C.ResourceIO m
-           => ResourceT m ()
-           -> C.Source m a
-           -> C.Source m a
-addCleanup cleanup src = src
-    { C.sourcePull = do
-        res <- C.sourcePull src
-        case res of
-            C.Closed -> cleanup >> return C.Closed
-            C.Open src' val -> return $ C.Open
-                (addCleanup cleanup src')
-                val
-    , C.sourceClose = do
-        C.sourceClose src
-        cleanup
-    }
+    return $ Response s version hs' body
